import time

from dictdiffer import diff

from .client import Client
from .alarm import Alarm
from .utils import filter_only, filter_exclude, resolved_dict


class CloudWatch(Client, object):

    def __init__(self, aws_access_key_id=None, aws_access_secret_key=None,
                 aws_session_token=None, aws_default_region=None, debug=None):
        super(CloudWatch, self).__init__(aws_access_key_id,
                                         aws_access_secret_key,
                                         aws_session_token,
                                         aws_default_region,
                                         debug)
        self.cloudwatch_client = self.session.client('cloudwatch')

    def remote_alarms(self, namespace, prefix):
        script_alarms = []
        human_alarms = []
        pager = self.cloudwatch_client.get_paginator('describe_alarms')
        for page in pager.paginate(AlarmNamePrefix=prefix):
            for alarm in page['MetricAlarms']:
                alarm = Alarm(name=alarm.get('AlarmName'), info=alarm)
                if alarm.namespace != namespace:
                    continue
                if alarm.is_created_by_script():
                    alarm = script_alarms.append(alarm)
                else:
                    human_alarms.append(alarm)
        return human_alarms, script_alarms

    def create_alarm(self, **kwargs):
        for _ in range(3):
            try:
                time.sleep(0.5)
                self.cloudwatch_client.put_metric_alarm(**kwargs)
            except Exception as e:
                print(e)
                continue
            break
        else:
            print('Exception when doing put_metric_alarm.')

    def create(self, instances, objects, namespace, prefix, default=None,
               only=None, exclude=None, sns={}, simulate=False):
        remote_alarms = self.remote_alarms(namespace=namespace, prefix=prefix)
        human_alarms, scripts_alarms = remote_alarms

        if only and len(only) > 0:
            instances = filter_only(instances, only)

        if exclude and len(exclude) > 0:
            instances = filter_exclude(instances, exclude)

        to_create_total = []
        to_update_scripted_total = []
        to_update_humans_total = []
        to_ignore_scripted_total = []
        to_ignore_humans_total = []

        for instance in instances:
            to_create = []
            to_update_scripted = []
            to_update_humans = []
            to_ignore_scripted = []
            to_ignore_humans = []
            all_alarms = objects.get('all') or []
            instance_name = instance.name
            filtered_instance = list(filter(lambda i: i in instance.name, objects.keys())) # noqa E501
            if len(filtered_instance) > 0:
                instance_name = max(filtered_instance, key=len)
            alarms = objects.get(instance_name) or []
            alarms_names = [a.name for a in alarms]
            all_alarms = [obj for obj in all_alarms if obj.name not in alarms_names] # noqa E501
            alarms = alarms + all_alarms
            if len(alarms) > 0:
                for alarm in alarms:

                    new_params = resolved_dict(name=alarm.name,
                                               instance=instance,
                                               namespace=namespace,
                                               original=alarm.dict(),
                                               default=default,
                                               prefix=prefix,
                                               sns=sns)

                    alarm = Alarm(name=alarm.name, info=new_params)

                    print('  - Checking alarm: {}'.format(alarm.name))

                    found1 = [a for a in human_alarms if a.name == alarm.name]
                    found2 = [a for a in scripts_alarms if a.name == alarm.name] # noqa E501

                    if found1 and len(found1) > 0:
                        found1 = found1[0]
                    if found2 and len(found2) > 0:
                        found2 = found2[0]

                    is_human = all([found1, not found2])

                    if "MaximumUsedTransactionIDs" in alarm.metric_name:
                        alarm.ok_actions = [sns['default']]
                        alarm.alarm_actions = [sns['default']]
                        alarm.threshold = 1000000000

                    if "HTTPCode_ELB_5XX_Count" in alarm.metric_name:
                        alarm.ok_actions = [sns['default']]
                        alarm.alarm_actions = [sns['default']]

                    if "5XX" in alarm.metric_name:
                        alarm.treat_missing_data = 'notBreaching'

                    if is_human:
                        alarm.threshold = found1.threshold
                        alarm.description = found1.description
                        alarm.evaluation_periods = found1.evaluation_periods
                        alarm.period = found1.period

                    if alarm.is_valid():
                        if is_human:
                            alarm.is_human = True
                            found1.is_human = True
                            dict1 = found1.dict()
                        else:
                            dict1 = found2.dict() if type(found2).__name__ == 'Alarm' else {}  # noqa E501

                        dict2 = alarm.dict()

                        if dict1 and dict2:
                            differences = list(diff(dict1, dict2))
                            if len(differences) > 0:
                                if is_human:
                                    to_update_humans.append(alarm)
                                else:
                                    to_update_scripted.append(alarm)
                                if self.debug:
                                    print('    - Update action. (Human alarm: %s)') % is_human  # noqa E501
                                    print('      - Diff:')
                                    for changes in differences:
                                        keys = changes[1]
                                        values = changes[2]
                                        print('        - %s | %s | %s') % (changes[0], keys, values)  # noqa E501
                            else:
                                if is_human:
                                    if self.debug:
                                        print('      - Already up to date.')
                                    to_ignore_humans.append(alarm)
                                    continue
                                else:
                                    if self.debug:
                                        print('      - Already up to date.')
                                    to_ignore_scripted.append(alarm)
                                    continue
                        else:
                            to_create.append(alarm)
                            if self.debug:
                                print('    - Create action.')
                                print('      - Params: {}'.format(alarm.dict()))  # noqa E501
                    else:
                        print('  - Invalid parameters.')
                        print('    - Params: {}'.format(alarm.dict()))

                    if self.debug:
                        print  # New line

                    if not simulate:
                        self.create_alarm(**alarm.dict())

                print('\n  - Create total: {}'.format(len(to_create)))
                print('  - Update total: (Scripted: {} | Humans: {})'.format(len(to_update_scripted),  # noqa E501
                                                                             len(to_update_humans)))  # noqa E501
                print('  - Ignore total: (Scripted: {} | Humans: {})'.format(len(to_ignore_scripted),  # noqa E501
                                                                             len(to_ignore_humans)))  # noqa E501

                to_create_total.append(to_create)
                to_update_scripted_total.append(to_update_scripted)
                to_update_humans_total.append(to_update_humans)
                to_ignore_scripted_total.append(to_ignore_scripted)
                to_ignore_humans_total.append(to_ignore_humans)

                print  # New line
            else:
                print('  - None')

        print('\nCreate total: {}'.format(sum([len(a) for a in to_create_total])))  # noqa E501
        print('Update total: (Scripted: {} | Humans: {})'.format(sum([len(a) for a in to_update_scripted_total]),  # noqa E501
                                                                 sum([len(a) for a in to_update_humans_total])))  # noqa E501
        print('Ignore total: (Scripted: {} | Humans: {})'.format(sum([len(a) for a in to_ignore_scripted_total]),  # noqa E501
                                                                 sum([len(a) for a in to_ignore_humans_total])))  # noqa E501



from .cloudwatch import CloudWatch


class ElastiCacheInstance:

    TYPE_TO_CORES = {
        "cache.t2.micro": 1,
        "cache.t2.small": 1,
        "cache.t2.medium": 2,
        "cache.m3.medium": 1,
        "cache.m3.large": 2,
        "cache.m3.xlarge": 4,
        "cache.m3.2xlarge": 8,
        "cache.m4.large": 2,
        "cache.m4.xlarge": 4,
        "cache.m4.2xlarge": 8,
        "cache.m4.4xlarge": 16,
        "cache.m4.10xlarge": 40,
        "cache.r3.large": 2,
        "cache.r3.xlarge": 4,
        "cache.r3.2xlarge": 8,
        "cache.r3.4xlarge": 16,
        "cache.r3.8xlarge": 32,
        "cache.r4.large": 2,
        "cache.r4.xlarge": 4,
        "cache.r4.2xlarge": 8,
        "cache.r4.4xlarge": 16,
        "cache.r4.8xlarge": 32,
        "cache.r4.16xlarge": 64,
    }

    def __init__(self, client, info):
        self.client = client
        self.arn = info.get('CacheClusterArn')
        self.name = info.get('CacheClusterId')
        self.engine = info.get('Engine')
        self.type = info.get('CacheNodeType')

    def __str__(self):
        return '(ElastiCacheInstance) Name: %s, Engine: %s, Type: %s' % (self.name, self.engine, self.type) # noqa E501

    def default_dimension_name(self):
        return 'CacheClusterId'

    def default_dimension_value(self):
        return self.name

    def default_dimensions(self):
        return [dict(Name=self.default_dimension_name(),
                     Value=self.default_dimension_value())]

    def cpu_utilization_threshold_modifier(self, threshold):
        if self.engine == 'redis':
            if self.type not in self.TYPE_TO_CORES:
                raise Exception('Unknown instance type %s.' % self.type)
            cores = self.TYPE_TO_CORES[self.type]
            return threshold / cores
        return threshold

    def dict(self):
        return {'CacheClusterArn': self.arn,
                'CacheClusterId': self.name}


class ElastiCache(CloudWatch, object):

    DEFAULT_NAMESPACE = 'AWS/ElastiCache'
    ALARM_NAME_PREFIX = 'Cache'

    def __init__(self, aws_access_key_id=None, aws_access_secret_key=None,
                 aws_session_token=None, aws_default_region=None, debug=None):
        super(ElastiCache, self).__init__(aws_access_key_id=aws_access_key_id,
                                          aws_access_secret_key=aws_access_secret_key, # noqa E501
                                          aws_session_token=aws_session_token,
                                          aws_default_region=aws_default_region, # noqa E501
                                          debug=debug)
        self.client = self.session.client('elasticache')

    def _describe_elastic_caches(self):
        caches = []
        pager = self.client.get_paginator('describe_cache_clusters')
        for page in pager.paginate():
            for i in page['CacheClusters']:
                caches.append(ElastiCacheInstance(self.client, i))
        return caches

    def list(self):
        return self._describe_elastic_caches()

    def remote_alarms(self, namespace=DEFAULT_NAMESPACE,
                      prefix=ALARM_NAME_PREFIX):
        namespace = namespace or ElastiCache.DEFAULT_NAMESPACE
        prefix = prefix or ElastiCache.ALARM_NAME_PREFIX
        return super(ElastiCache, self).remote_alarms(namespace=namespace,
                                                      prefix=prefix)

    def create(self, objects, namespace=DEFAULT_NAMESPACE,
               prefix=ALARM_NAME_PREFIX, default=None, only=None,
               exclude=None, sns={}, simulate=False, engine=None):
        if exclude is not None and only is not None:
            raise "Exlude and Only option are mutually exclusive."

        instances = self._describe_elastic_caches()

        redis_instances = [i for i in instances if i.engine == 'redis']
        memcached_instances = [i for i in instances if i not in redis_instances] # noqa E501

        if engine is None or engine == 'redis':
            super(ElastiCache, self).create(instances=redis_instances,
                                            objects=objects,
                                            namespace=namespace,
                                            prefix='%s/redis' % prefix,
                                            default=default,
                                            only=only,
                                            exclude=exclude,
                                            sns=sns,
                                            simulate=simulate)

        if engine is None or engine == 'memcached':
            super(ElastiCache, self).create(instances=memcached_instances,
                                            objects=objects,
                                            namespace=namespace,
                                            prefix='%s/memcached' % prefix,
                                            default=default,
                                            only=only,
                                            exclude=exclude,
                                            sns=sns,
                                            simulate=simulate)



# -*- coding: utf-8 -*-
import json
import os
import re
import yaml
from os.path import expanduser

import click

from .default_alarm import DefaultAlarm
from .alarm import Alarm
from .alb import ALB
from .pgbouncer import PGBouncer
from .elb import ELB
from .rds import RDS
from .ec2 import EC2
from .kinesis import Kinesis
from .elastic_cache import ElastiCache
from .utils import json_serializer

# Fix Python 2.x vs 3.x.
try:
    bool(type(unicode))
    UNICODE_TYPE = unicode
except NameError:
    UNICODE_TYPE = str


CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])
ALB_TMP_FILE = "{}/{}/{}".format(expanduser("~"),
                                 '.cwam',
                                 'alb.template.yml')
ELB_TMP_FILE = "{}/{}/{}".format(expanduser("~"),
                                 '.cwam',
                                 'elb.template.yml')
RDS_TMP_FILE = "{}/{}/{}".format(expanduser("~"),
                                 '.cwam',
                                 'rds.template.yml')
KINESIS_TMP_FILE = "{}/{}/{}".format(expanduser("~"),
                                     '.cwam',
                                     'kinesis.template.yml')
ELASTIC_CACHE_TMP_FILE = "{}/{}/{}".format(expanduser("~"),
                                           '.cwam',
                                           'elastic_cache.template.yml')
EC2_TMP_FILE = "{}/{}/{}".format(expanduser("~"),
                                 '.cwam',
                                 'ec2.template.yml')
PGBOUNCER_TMP_FILE = "{}/{}/{}".format(expanduser("~"),
                                       '.cwam',
                                       'pgbouncer.template.yml')


def json_dumps(dict, pretty=False):
    if pretty:
        return json.dumps(dict, indent=2, default=json_serializer)
    else:
        return json.dumps(dict, default=json_serializer)


def parse_yml(ctx, path):
    with open(path, 'r') as stream:
        try:
            content = yaml.load(stream)
        except yaml.YAMLError as e:
            ctx.fail(e)
        return content


def parse_exclude_only(infos):
    if infos:
        d = [dict(key=i['key'], regexp=re.compile(i['regexp'])) for i in infos]
        return d
    else:
        return infos


def parse_default_alarm(namespace, objects):
    alarms = {}
    if objects:
        for k, v in objects.items():
            alarms[k] = DefaultAlarm(namespace=namespace, info=v)
    alarms['sns'] = objects.get('sns')
    return alarms


def parse_alarms(namespace, objects):
    alarms = {}
    for k, v in objects.items():
        alarms_obj = []
        for alarm in v:
            name = next(iter(alarm))
            alarms_obj.append(Alarm(name=name,
                                    namespace=namespace,
                                    info=alarm.get(name)))
        alarms[k] = alarms_obj
    return alarms


def parse_alarms_yml(ctx, command, conf):
    if os.path.isfile(conf):
        namespace = parse_yml(ctx, conf).get(command).get('namespace')
        alarms = parse_yml(ctx, conf).get(command).get('alarms')
        return namespace, alarms
    else:
        ctx.fail('Conf file not found. Make sure --template is a valid path.')


@click.version_option('0.1.0')
@click.group(context_settings=CONTEXT_SETTINGS)
@click.option('--debug', '-d', flag_value=True, default=False,
              help='Debug mode.')
@click.option('--pretty', '-p', flag_value=True, default=False,
              help='Prettify JSON output.')
@click.option('--aws-access-key-id', '-k', type=UNICODE_TYPE,
              envvar='AWS_ACCESS_KEY_ID',
              help='AWS Access Key ID.')
@click.option('--aws-access-secret-key', '-s', type=UNICODE_TYPE,
              envvar='AWS_SECRET_ACCESS_KEY',
              help='AWS Secret Access Key.')
@click.option('--aws-session-token', '-t', type=UNICODE_TYPE,
              envvar='AWS_SESSION_TOKEN',
              help='AWS Secret Access Key.')
@click.option('--aws_default_region', '-r', type=UNICODE_TYPE,
              envvar='AWS_DEFAULT_REGION',
              default='us-east-1',
              help='AWS Region.')
@click.pass_context
def main(ctx, debug, pretty, aws_access_key_id, aws_access_secret_key,
         aws_session_token, aws_default_region):
    ctx.obj = {}
    ctx.obj['PRETTY'] = pretty if pretty else None
    ctx.obj['DEBUG'] = debug if debug else None

    if aws_access_key_id:
        ctx.obj['AWS_ACCESS_KEY_ID'] = aws_access_key_id
    else:
        ctx.obj['AWS_ACCESS_KEY_ID'] = None

    if aws_access_secret_key:
        ctx.obj['AWS_SECRET_ACCESS_KEY'] = aws_access_secret_key
    else:
        ctx.obj['AWS_SECRET_ACCESS_KEY'] = None

    if aws_session_token:
        ctx.obj['AWS_SESSION_TOKEN'] = aws_session_token
    else:
        ctx.obj['AWS_SESSION_TOKEN'] = None

    if aws_default_region:
        ctx.obj['AWS_DEFAULT_REGION'] = aws_default_region
    else:
        ctx.obj['AWS_DEFAULT_REGION'] = None


@main.group()
@click.pass_context
def alb(ctx):
    pass


@alb.command(name='list')  # noqa: F811
@click.pass_context
def alb_list(ctx):
    """List ALB."""
    instances = ALB(aws_access_key_id=ctx.obj['AWS_ACCESS_KEY_ID'],
                    aws_access_secret_key=ctx.obj['AWS_SECRET_ACCESS_KEY'],
                    aws_session_token=ctx.obj['AWS_SESSION_TOKEN'],
                    aws_default_region=ctx.obj['AWS_DEFAULT_REGION'],
                    debug=ctx.obj['DEBUG']).list()
    for instance in instances:
        click.echo(instance)


@alb.command(name='create')  # noqa: F811
@click.pass_context
@click.option('--template', '-t', type=UNICODE_TYPE,
              default=ALB_TMP_FILE,
              help='Path to template file. Default: {}.'.format(ALB_TMP_FILE))
@click.option('--simulate', '-s', is_flag=True, default=False,
              help='Simulate only. Do not take actions')
def alb_create(ctx, template, simulate):
    """Create alarms configured in --template file"""
    if os.path.isfile(template):
        template = parse_yml(ctx, template)['alb']
        namespace = template.get('namespace')
        prefix = template.get('prefix')
        only = template.get('only')
        exclude = template.get('exclude')
        sns = template.get('sns')
        default = template.get('default')
        alarms = template.get('alarms')
    else:
        ctx.fail('Conf file not found. Make sure --template is a valid path.')

    if len(alarms) > 0:
        alb = ALB(aws_access_key_id=ctx.obj['AWS_ACCESS_KEY_ID'],
                  aws_access_secret_key=ctx.obj['AWS_SECRET_ACCESS_KEY'],
                  aws_session_token=ctx.obj['AWS_SESSION_TOKEN'],
                  aws_default_region=ctx.obj['AWS_DEFAULT_REGION'],
                  debug=ctx.obj['DEBUG'])
        alb.create(objects=parse_alarms(namespace, alarms),
                   namespace=namespace,
                   prefix=prefix,
                   default=parse_default_alarm(namespace, default),
                   only=parse_exclude_only(only),
                   exclude=parse_exclude_only(exclude),
                   sns=sns,
                   simulate=simulate)
    else:
        click.echo('No alarms found.')


@alb.command(name='local-alarms')  # noqa: F811
@click.pass_context
@click.option('--template', '-t', type=UNICODE_TYPE,
              default=ALB_TMP_FILE,
              help='Path to template file. Default: {}.'.format(ALB_TMP_FILE))
def alb_local_alarms(ctx, template):
    namespace, alarms = parse_alarms_yml(ctx, 'alb', template)
    for k, v in parse_alarms(namespace, alarms).items():
        click.echo(k)
        for alarm in v:
            click.echo(str(alarm))


@alb.command(name='remote-alarms')  # noqa: F811
@click.pass_context
@click.option('--template', '-t', type=UNICODE_TYPE,
              default=ALB_TMP_FILE,
              help='Path to template file. Default: {}.'.format(ALB_TMP_FILE))
@click.option('--no-human', '-h', is_flag=True, default=False,
              help='Show only human alarms.')
@click.option('--no-script', '-s', is_flag=True, default=False,
              help='Show only script alarms.')
def alb_remote_alarms(ctx, template, no_human, no_script):
    """List alarms configured on AWS"""
    if os.path.isfile(template):
        template = parse_yml(ctx, template)['alb']
        namespace = template.get('namespace')
        prefix = template.get('prefix')
    else:
        namespace = None
        prefix = None

    alb = ALB(aws_access_key_id=ctx.obj['AWS_ACCESS_KEY_ID'],
              aws_access_secret_key=ctx.obj['AWS_SECRET_ACCESS_KEY'],
              aws_session_token=ctx.obj['AWS_SESSION_TOKEN'],
              aws_default_region=ctx.obj['AWS_DEFAULT_REGION'],
              debug=ctx.obj['DEBUG'])
    human_alarms, script_alarms = alb.remote_alarms(namespace=namespace,
                                                    prefix=prefix)

    if not no_human:
        click.echo('Human alarms.')
        if len(human_alarms) > 0:
            for alarm in human_alarms:
                click.echo(str(alarm))
        else:
            click.echo('None.')

    if not no_script:
        click.echo('Script alarms.')
        if len(script_alarms) > 0:
            for alarm in script_alarms:
                click.echo(str(alarm))
        else:
            click.echo('None.')


@main.group()
@click.pass_context
def elb(ctx):
    pass


@elb.command(name='list')  # noqa: F811
@click.pass_context
def elb_list(ctx):
    """List ELB."""
    instances = ELB(aws_access_key_id=ctx.obj['AWS_ACCESS_KEY_ID'],
                    aws_access_secret_key=ctx.obj['AWS_SECRET_ACCESS_KEY'],
                    aws_session_token=ctx.obj['AWS_SESSION_TOKEN'],
                    aws_default_region=ctx.obj['AWS_DEFAULT_REGION'],
                    debug=ctx.obj['DEBUG']).list()
    for instance in instances:
        click.echo(instance)


@elb.command(name='create')  # noqa: F811
@click.pass_context
@click.option('--template', '-t', type=UNICODE_TYPE,
              default=ELB_TMP_FILE,
              help='Path to template file. Default: {}.'.format(ELB_TMP_FILE))
@click.option('--simulate', '-s', is_flag=True, default=False,
              help='Simulate only. Do not take actions')
def elb_create(ctx, template, simulate):
    """Create alarms configured in --template file"""
    if os.path.isfile(template):
        template = parse_yml(ctx, template)['elb']
        namespace = template.get('namespace')
        prefix = template.get('prefix')
        only = template.get('only')
        exclude = template.get('exclude')
        sns = template.get('sns')
        default = template.get('default')
        alarms = template.get('alarms')
    else:
        ctx.fail('Conf file not found. Make sure --template is a valid path.')

    if len(alarms) > 0:
        elb = ELB(aws_access_key_id=ctx.obj['AWS_ACCESS_KEY_ID'],
                  aws_access_secret_key=ctx.obj['AWS_SECRET_ACCESS_KEY'],
                  aws_session_token=ctx.obj['AWS_SESSION_TOKEN'],
                  aws_default_region=ctx.obj['AWS_DEFAULT_REGION'],
                  debug=ctx.obj['DEBUG'])
        elb.create(objects=parse_alarms(namespace, alarms),
                   namespace=namespace,
                   prefix=prefix,
                   default=parse_default_alarm(namespace, default),
                   only=parse_exclude_only(only),
                   exclude=parse_exclude_only(exclude),
                   sns=sns,
                   simulate=simulate)
    else:
        click.echo('No alarms found.')


@elb.command(name='local-alarms')  # noqa: F811
@click.pass_context
@click.option('--template', '-t', type=UNICODE_TYPE,
              default=ELB_TMP_FILE,
              help='Path to template file. Default: {}.'.format(ELB_TMP_FILE))
def elb_local_alarms(ctx, template):
    namespace, alarms = parse_alarms_yml(ctx, 'elb', template)
    for k, v in parse_alarms(namespace, alarms).items():
        click.echo(k)
        for alarm in v:
            click.echo(str(alarm))


@elb.command(name='remote-alarms')  # noqa: F811
@click.pass_context
@click.option('--template', '-t', type=UNICODE_TYPE,
              default=ELB_TMP_FILE,
              help='Path to template file. Default: {}.'.format(ELB_TMP_FILE))
@click.option('--no-human', '-h', is_flag=True, default=False,
              help='Show only human alarms.')
@click.option('--no-script', '-s', is_flag=True, default=False,
              help='Show only script alarms.')
def elb_remote_alarms(ctx, template, no_human, no_script):
    """List alarms configured on AWS"""
    if os.path.isfile(template):
        template = parse_yml(ctx, template)['elb']
        namespace = template.get('namespace')
        prefix = template.get('prefix')
    else:
        namespace = None
        prefix = None

    elb = ELB(aws_access_key_id=ctx.obj['AWS_ACCESS_KEY_ID'],
              aws_access_secret_key=ctx.obj['AWS_SECRET_ACCESS_KEY'],
              aws_session_token=ctx.obj['AWS_SESSION_TOKEN'],
              aws_default_region=ctx.obj['AWS_DEFAULT_REGION'],
              debug=ctx.obj['DEBUG'])
    human_alarms, script_alarms = elb.remote_alarms(namespace=namespace,
                                                    prefix=prefix)

    if not no_human:
        click.echo('Human alarms.')
        if len(human_alarms) > 0:
            for alarm in human_alarms:
                click.echo(str(alarm))
        else:
            click.echo('None.')

    if not no_script:
        click.echo('Script alarms.')
        if len(script_alarms) > 0:
            for alarm in script_alarms:
                click.echo(str(alarm))
        else:
            click.echo('None.')


@main.group()
@click.pass_context
def rds(ctx):
    pass


@rds.command(name='list')  # noqa: F811
@click.pass_context
def rds_list(ctx):
    """List RDS clusters."""
    instances = RDS(aws_access_key_id=ctx.obj['AWS_ACCESS_KEY_ID'],
                    aws_access_secret_key=ctx.obj['AWS_SECRET_ACCESS_KEY'],
                    aws_session_token=ctx.obj['AWS_SESSION_TOKEN'],
                    aws_default_region=ctx.obj['AWS_DEFAULT_REGION'],
                    debug=ctx.obj['DEBUG']).list()
    for instance in instances:
        click.echo(instance)


@rds.command(name='create')  # noqa: F811
@click.pass_context
@click.option('--template', '-t', type=UNICODE_TYPE,
              default=RDS_TMP_FILE,
              help='Path to template file. Default: {}.'.format(RDS_TMP_FILE))
@click.option('--simulate', '-s', is_flag=True, default=False,
              help='Simulate only. Do not take actions')
def rds_create(ctx, template, simulate):
    """Create alarms configured in --template file"""
    if os.path.isfile(template):
        template = parse_yml(ctx, template)['rds']
        namespace = template.get('namespace')
        prefix = template.get('prefix')
        only = template.get('only')
        exclude = template.get('exclude')
        sns = template.get('sns')
        default = template.get('default')
        alarms = template.get('alarms')
    else:
        ctx.fail('Conf file not found. Make sure --template is a valid path.')

    if len(alarms) > 0:
        rds = RDS(aws_access_key_id=ctx.obj['AWS_ACCESS_KEY_ID'],
                  aws_access_secret_key=ctx.obj['AWS_SECRET_ACCESS_KEY'],
                  aws_session_token=ctx.obj['AWS_SESSION_TOKEN'],
                  aws_default_region=ctx.obj['AWS_DEFAULT_REGION'],
                  debug=ctx.obj['DEBUG'])
        rds.create(objects=parse_alarms(namespace, alarms),
                   namespace=namespace,
                   prefix=prefix,
                   default=parse_default_alarm(namespace, default),
                   only=parse_exclude_only(only),
                   exclude=parse_exclude_only(exclude),
                   sns=sns,
                   simulate=simulate)
    else:
        click.echo('No alarms found.')


@rds.command(name='local-alarms')  # noqa: F811
@click.pass_context
@click.option('--template', '-t', type=UNICODE_TYPE,
              default=RDS_TMP_FILE,
              help='Path to template file. Default: {}.'.format(RDS_TMP_FILE))
def rds_local_alarms(ctx, template):
    namespace, alarms = parse_alarms_yml(ctx, 'rds', template)
    for k, v in parse_alarms(namespace, alarms).items():
        click.echo(k)
        for alarm in v:
            click.echo(str(alarm))


@rds.command(name='remote-alarms')  # noqa: F811
@click.pass_context
@click.option('--template', '-t', type=UNICODE_TYPE,
              default=RDS_TMP_FILE,
              help='Path to template file. Default: {}.'.format(RDS_TMP_FILE))
@click.option('--no-human', '-h', is_flag=True, default=False,
              help='Show only human alarms.')
@click.option('--no-script', '-s', is_flag=True, default=False,
              help='Show only script alarms.')
def rds_remote_alarms(ctx, template, no_human, no_script):
    """List alarms configured on AWS"""
    if os.path.isfile(template):
        template = parse_yml(ctx, template)['rds']
        namespace = template.get('namespace')
        prefix = template.get('prefix')
    else:
        namespace = None
        prefix = None

    rds = RDS(aws_access_key_id=ctx.obj['AWS_ACCESS_KEY_ID'],
              aws_access_secret_key=ctx.obj['AWS_SECRET_ACCESS_KEY'],
              aws_session_token=ctx.obj['AWS_SESSION_TOKEN'],
              aws_default_region=ctx.obj['AWS_DEFAULT_REGION'],
              debug=ctx.obj['DEBUG'])
    human_alarms, script_alarms = rds.remote_alarms(namespace=namespace,
                                                    prefix=prefix)

    if not no_human:
        click.echo('Human alarms.')
        if len(human_alarms) > 0:
            for alarm in human_alarms:
                click.echo(str(alarm))
        else:
            click.echo('None.')

    if not no_script:
        click.echo('Script alarms.')
        if len(script_alarms) > 0:
            for alarm in script_alarms:
                click.echo(str(alarm))
        else:
            click.echo('None.')


@main.group()
@click.pass_context
def pgbouncer(ctx):
    pass


@pgbouncer.command(name='list')
@click.pass_context
def pgbouncer_list(ctx):
    """List PGBouncer servers."""
    instances = PGBouncer(aws_access_key_id=ctx.obj['AWS_ACCESS_KEY_ID'],
                          aws_access_secret_key=ctx.obj['AWS_SECRET_ACCESS_KEY'],  # noqa E501
                          aws_session_token=ctx.obj['AWS_SESSION_TOKEN'],
                          aws_default_region=ctx.obj['AWS_DEFAULT_REGION'],
                          debug=ctx.obj['DEBUG']).list()
    for instance in instances:
        click.echo(instance)


@pgbouncer.command(name='create')
@click.pass_context
@click.option('--template', '-t', type=UNICODE_TYPE,
              default=PGBOUNCER_TMP_FILE,
              help='Path to template file. Default: {}.'.format(PGBOUNCER_TMP_FILE))  # noqa E501
@click.option('--simulate', '-s', is_flag=True, default=False,
              help='Simulate only. Do not take actions')
def pgbouncer_create(ctx, template, simulate):
    """Create alarms configured in --template file"""
    if os.path.isfile(template):
        template = parse_yml(ctx, template)['pgbouncer']
        namespace = template.get('namespace')
        prefix = template.get('prefix')
        only = template.get('only')
        exclude = template.get('exclude')
        sns = template.get('sns')
        default = template.get('default')
        alarms = template.get('alarms')
    else:
        ctx.fail('Conf file not found. Make sure --template is a valid path.')

    if len(alarms) > 0:
        pgbouncer = PGBouncer(aws_access_key_id=ctx.obj['AWS_ACCESS_KEY_ID'],
                              aws_access_secret_key=ctx.obj['AWS_SECRET_ACCESS_KEY'],  # noqa E501
                              aws_session_token=ctx.obj['AWS_SESSION_TOKEN'],
                              aws_default_region=ctx.obj['AWS_DEFAULT_REGION'],
                              debug=ctx.obj['DEBUG'])
        pgbouncer.create(objects=parse_alarms(namespace, alarms),
                         namespace=namespace,
                         prefix=prefix,
                         default=parse_default_alarm(namespace, default),
                         only=parse_exclude_only(only),
                         exclude=parse_exclude_only(exclude),
                         sns=sns, simulate=simulate)
    else:
        click.echo('No alarms found.')


@pgbouncer.command(name='local-alarms')  # noqa: F811
@click.pass_context
@click.option('--template', '-t', type=UNICODE_TYPE,
              default=PGBOUNCER_TMP_FILE,
              help='Path to template file. Default: {}.'.format(PGBOUNCER_TMP_FILE))  # noqa E501
def pgbouncer_local_alarms(ctx, template):
    namespace, alarms = parse_alarms_yml(ctx, 'pgbouncer', template)
    for k, v in parse_alarms(namespace, alarms).items():
        click.echo(k)
        for alarm in v:
            click.echo(str(alarm))


@pgbouncer.command(name='remote-alarms')  # noqa: F811
@click.pass_context
@click.option('--template', '-t', type=UNICODE_TYPE,
              default=PGBOUNCER_TMP_FILE,
              help='Path to template file. Default: {}.'.format(PGBOUNCER_TMP_FILE))  # noqa E501
@click.option('--no-human', '-h', is_flag=True, default=False,
              help='Show only human alarms.')
@click.option('--no-script', '-s', is_flag=True, default=False,
              help='Show only script alarms.')
def pgbouncer_remote_alarms(ctx, template, no_human, no_script):
    """List alarms configured on AWS"""
    if os.path.isfile(template):
        template = parse_yml(ctx, template)['pgbouncer']
        namespace = template.get('namespace')
        prefix = template.get('prefix')
    else:
        namespace = None
        prefix = None

    pgbouncer = PGBouncer(aws_access_key_id=ctx.obj['AWS_ACCESS_KEY_ID'],
                          aws_access_secret_key=ctx.obj['AWS_SECRET_ACCESS_KEY'],  # noqa E501
                          aws_session_token=ctx.obj['AWS_SESSION_TOKEN'],
                          aws_default_region=ctx.obj['AWS_DEFAULT_REGION'],
                          debug=ctx.obj['DEBUG'])
    human_alarms, script_alarms = pgbouncer.remote_alarms(namespace=namespace,
                                                          prefix=prefix)

    if not no_human:
        click.echo('Human alarms.')
        if len(human_alarms) > 0:
            for alarm in human_alarms:
                click.echo(str(alarm))
        else:
            click.echo('None.')

    if not no_script:
        click.echo('Script alarms.')
        if len(script_alarms) > 0:
            for alarm in script_alarms:
                click.echo(str(alarm))
        else:
            click.echo('None.')


@main.group()
@click.pass_context
def elastic_cache(ctx):
    pass


@elastic_cache.command(name='list')  # noqa: F811
@click.pass_context
def elastic_cache_list(ctx):
    """List ElastiCache clusters."""
    instances = ElastiCache(aws_access_key_id=ctx.obj['AWS_ACCESS_KEY_ID'],
                            aws_access_secret_key=ctx.obj['AWS_SECRET_ACCESS_KEY'],  # noqa E501
                            aws_session_token=ctx.obj['AWS_SESSION_TOKEN'],
                            aws_default_region=ctx.obj['AWS_DEFAULT_REGION'],
                            debug=ctx.obj['DEBUG']).list()
    for instance in instances:
        click.echo(instance)


@elastic_cache.command(name='create')  # noqa: F811
@click.pass_context
@click.option('--template', '-t', type=UNICODE_TYPE,
              default=ELASTIC_CACHE_TMP_FILE,
              help='Path to template file. Default: {}.'.format(ELASTIC_CACHE_TMP_FILE))  # noqa E501
@click.option('--simulate', '-s', is_flag=True, default=False,
              help='Simulate only. Do not take actions')
def elastic_cache_create(ctx, template, simulate):
    """Create alarms configured in --template file"""
    if os.path.isfile(template):
        template = parse_yml(ctx, template)['elastic_caches']
        namespace = template.get('namespace')
        prefix = template.get('prefix')
        only = template.get('only')
        exclude = template.get('exclude')
        sns = template.get('sns')
        default = template.get('default')
        alarms = template.get('alarms')
        engine = template.get('engine')
    else:
        ctx.fail('Conf file not found. Make sure --template is a valid path.')

    if len(alarms) > 0:
        elastic_cache = ElastiCache(aws_access_key_id=ctx.obj['AWS_ACCESS_KEY_ID'],  # noqa E501
                                    aws_access_secret_key=ctx.obj['AWS_SECRET_ACCESS_KEY'],  # noqa E501
                                    aws_session_token=ctx.obj['AWS_SESSION_TOKEN'],  # noqa E501
                                    aws_default_region=ctx.obj['AWS_DEFAULT_REGION'],  # noqa E501
                                    debug=ctx.obj['DEBUG'])
        elastic_cache.create(objects=parse_alarms(namespace, alarms),
                             namespace=namespace,
                             prefix=prefix,
                             default=parse_default_alarm(namespace, default),
                             only=parse_exclude_only(only),
                             exclude=parse_exclude_only(exclude),
                             sns=sns,
                             simulate=simulate,
                             engine=engine)
    else:
        click.echo('No alarms found.')


@elastic_cache.command(name='local-alarms')  # noqa: F811
@click.pass_context
@click.option('--template', '-t', type=UNICODE_TYPE,
              default=ELASTIC_CACHE_TMP_FILE,
              help='Path to template file. Default: {}.'.format(ELASTIC_CACHE_TMP_FILE))  # noqa E501
def elastic_cache_local_alarms(ctx, template):
    namespace, alarms = parse_alarms_yml(ctx, 'elastic_caches', template)
    for k, v in parse_alarms(namespace, alarms).items():
        click.echo(k)
        for alarm in v:
            click.echo(str(alarm))


@elastic_cache.command(name='remote-alarms')  # noqa: F811
@click.pass_context
@click.option('--template', '-t', type=UNICODE_TYPE,
              default=ELASTIC_CACHE_TMP_FILE,
              help='Path to template file. Default: {}.'.format(ELASTIC_CACHE_TMP_FILE))  # noqa E501
@click.option('--no-human', '-h', is_flag=True, default=False,
              help='Show only human alarms.')
@click.option('--no-script', '-s', is_flag=True, default=False,
              help='Show only script alarms.')
def elastic_cache_remote_alarms(ctx, template, no_human, no_script):
    """List alarms configured on AWS"""
    if os.path.isfile(template):
        template = parse_yml(ctx, template)['elastic_caches']
        namespace = template.get('namespace')
        prefix = template.get('prefix')
    else:
        namespace = None
        prefix = None

    elastic_cache = ElastiCache(aws_access_key_id=ctx.obj['AWS_ACCESS_KEY_ID'],  # noqa E501
                                aws_access_secret_key=ctx.obj['AWS_SECRET_ACCESS_KEY'],  # noqa E501
                                aws_session_token=ctx.obj['AWS_SESSION_TOKEN'],  # noqa E501
                                aws_default_region=ctx.obj['AWS_DEFAULT_REGION'],  # noqa E501
                                debug=ctx.obj['DEBUG'])
    human_alarms, script_alarms = elastic_cache.remote_alarms(namespace=namespace,  # noqa E501
                                                              prefix=prefix)

    if not no_human:
        click.echo('Human alarms.')
        if len(human_alarms) > 0:
            for alarm in human_alarms:
                click.echo(str(alarm))
        else:
            click.echo('None.')

    if not no_script:
        click.echo('Script alarms.')
        if len(script_alarms) > 0:
            for alarm in script_alarms:
                click.echo(str(alarm))
        else:
            click.echo('None.')


@main.group()
@click.pass_context
def ec2(ctx):
    pass


@ec2.command(name='list')  # noqa: F811
@click.pass_context
def ec2_list(ctx):
    """List EC2 clusters."""
    instances = EC2(aws_access_key_id=ctx.obj['AWS_ACCESS_KEY_ID'],
                    aws_access_secret_key=ctx.obj['AWS_SECRET_ACCESS_KEY'],
                    aws_session_token=ctx.obj['AWS_SESSION_TOKEN'],
                    aws_default_region=ctx.obj['AWS_DEFAULT_REGION'],
                    debug=ctx.obj['DEBUG']).list()
    for instance in instances:
        click.echo(instance)


@ec2.command(name='create')  # noqa: F811
@click.pass_context
@click.option('--template', '-t', type=UNICODE_TYPE,
              default=EC2_TMP_FILE,
              help='Path to template file. Default: {}.'.format(EC2_TMP_FILE))
@click.option('--simulate', '-s', is_flag=True, default=False,
              help='Simulate only. Do not take actions')
def ec2_create(ctx, template, simulate):
    """Create alarms configured in --template file"""
    if os.path.isfile(template):
        template = parse_yml(ctx, template)['ec2']
        namespace = template.get('namespace')
        prefix = template.get('prefix')
        only = template.get('only')
        exclude = template.get('exclude')
        sns = template.get('sns')
        default = template.get('default')
        alarms = template.get('alarms')
    else:
        ctx.fail('Conf file not found. Make sure --template is a valid path.')

    if len(alarms) > 0:
        ec2 = EC2(aws_access_key_id=ctx.obj['AWS_ACCESS_KEY_ID'],
                  aws_access_secret_key=ctx.obj['AWS_SECRET_ACCESS_KEY'],
                  aws_session_token=ctx.obj['AWS_SESSION_TOKEN'],
                  aws_default_region=ctx.obj['AWS_DEFAULT_REGION'],
                  debug=ctx.obj['DEBUG'])
        ec2.create(objects=parse_alarms(namespace, alarms),
                   namespace=namespace,
                   prefix=prefix,
                   default=parse_default_alarm(namespace, default),
                   only=parse_exclude_only(only),
                   exclude=parse_exclude_only(exclude),
                   sns=sns,
                   simulate=simulate)
    else:
        click.echo('No alarms found.')


@ec2.command(name='local-alarms')  # noqa: F811
@click.pass_context
@click.option('--template', '-t', type=UNICODE_TYPE,
              default=EC2_TMP_FILE,
              help='Path to template file. Default: {}.'.format(EC2_TMP_FILE))
def ec2_local_alarms(ctx, template):
    namespace, alarms = parse_alarms_yml(ctx, 'ec2', template)
    for k, v in parse_alarms(namespace, alarms).items():
        click.echo(k)
        for alarm in v:
            click.echo(str(alarm))


@ec2.command(name='remote-alarms')  # noqa: F811
@click.pass_context
@click.option('--template', '-t', type=UNICODE_TYPE,
              default=EC2_TMP_FILE,
              help='Path to template file. Default: {}.'.format(EC2_TMP_FILE))
@click.option('--no-human', '-h', is_flag=True, default=False,
              help='Show only human alarms.')
@click.option('--no-script', '-s', is_flag=True, default=False,
              help='Show only script alarms.')
def ec2_remote_alarms(ctx, template, no_human, no_script):
    """List alarms configured on AWS"""
    if os.path.isfile(template):
        template = parse_yml(ctx, template)['ec2']
        namespace = template.get('namespace')
        prefix = template.get('prefix')
    else:
        namespace = None
        prefix = None

    ec2 = EC2(aws_access_key_id=ctx.obj['AWS_ACCESS_KEY_ID'],
              aws_access_secret_key=ctx.obj['AWS_SECRET_ACCESS_KEY'],
              aws_session_token=ctx.obj['AWS_SESSION_TOKEN'],
              aws_default_region=ctx.obj['AWS_DEFAULT_REGION'],
              debug=ctx.obj['DEBUG'])
    human_alarms, script_alarms = ec2.remote_alarms(namespace=namespace,
                                                    prefix=prefix)

    if not no_human:
        click.echo('Human alarms.')
        if len(human_alarms) > 0:
            for alarm in human_alarms:
                click.echo(str(alarm))
        else:
            click.echo('None.')

    if not no_script:
        click.echo('Script alarms.')
        if len(script_alarms) > 0:
            for alarm in script_alarms:
                click.echo(str(alarm))
        else:
            click.echo('None.')


@main.group()
@click.pass_context
def kinesis(ctx):
    pass


@kinesis.command(name='list')  # noqa: F811
@click.pass_context
def kinesis_list(ctx):
    """List Kinesis streams."""
    streams = Kinesis(aws_access_key_id=ctx.obj['AWS_ACCESS_KEY_ID'],
                      aws_access_secret_key=ctx.obj['AWS_SECRET_ACCESS_KEY'],
                      aws_session_token=ctx.obj['AWS_SESSION_TOKEN'],
                      aws_default_region=ctx.obj['AWS_DEFAULT_REGION'],
                      debug=ctx.obj['DEBUG']).list()
    for streams in streams:
        click.echo(streams)


@kinesis.command(name='create')  # noqa: F811
@click.pass_context
@click.option('--template', '-t', type=UNICODE_TYPE,
              default=KINESIS_TMP_FILE,
              help='Path to template file. Default: {}.'.format(KINESIS_TMP_FILE))  # noqa E501
@click.option('--simulate', '-s', is_flag=True, default=False,
              help='Simulate only. Do not take actions')
def kinesis_create(ctx, template, simulate):
    """Create alarms configured in --template file"""
    if os.path.isfile(template):
        template = parse_yml(ctx, template)['kinesis']
        namespace = template.get('namespace')
        prefix = template.get('prefix')
        only = template.get('only')
        exclude = template.get('exclude')
        sns = template.get('sns')
        default = template.get('default')
        alarms = template.get('alarms')
    else:
        ctx.fail('Conf file not found. Make sure --template is a valid path.')

    if len(alarms) > 0:
        kinesis = Kinesis(aws_access_key_id=ctx.obj['AWS_ACCESS_KEY_ID'],
                          aws_access_secret_key=ctx.obj['AWS_SECRET_ACCESS_KEY'],  # noqa E501
                          aws_session_token=ctx.obj['AWS_SESSION_TOKEN'],
                          aws_default_region=ctx.obj['AWS_DEFAULT_REGION'],
                          debug=ctx.obj['DEBUG'])
        kinesis.create(objects=parse_alarms(namespace, alarms),
                       namespace=namespace,
                       prefix=prefix,
                       default=parse_default_alarm(namespace, default),
                       only=parse_exclude_only(only),
                       exclude=parse_exclude_only(exclude),
                       sns=sns,
                       simulate=simulate)
    else:
        click.echo('No alarms found.')


@kinesis.command(name='local-alarms')  # noqa: F811
@click.pass_context
@click.option('--template', '-t', type=UNICODE_TYPE,
              default=KINESIS_TMP_FILE,
              help='Path to template file. Default: {}.'.format(KINESIS_TMP_FILE))  # noqa E501
def kinesis_local_alarms(ctx, template):
    namespace, alarms = parse_alarms_yml(ctx, 'kinesis', template)
    for k, v in parse_alarms(namespace, alarms).items():
        click.echo(k)
        for alarm in v:
            click.echo(str(alarm))


@kinesis.command(name='remote-alarms')  # noqa: F811
@click.pass_context
@click.option('--template', '-t', type=UNICODE_TYPE,
              default=KINESIS_TMP_FILE,
              help='Path to template file. Default: {}.'.format(KINESIS_TMP_FILE))  # noqa E501
@click.option('--no-human', '-h', is_flag=True, default=False,
              help='Show only human alarms.')
@click.option('--no-script', '-s', is_flag=True, default=False,
              help='Show only script alarms.')
def kinesis_remote_alarms(ctx, template, no_human, no_script):
    """List alarms configured on AWS"""
    if os.path.isfile(template):
        template = parse_yml(ctx, template)['kinesis']
        namespace = template.get('namespace')
        prefix = template.get('prefix')
    else:
        namespace = None
        prefix = None

    kinesis = Kinesis(aws_access_key_id=ctx.obj['AWS_ACCESS_KEY_ID'],
                      aws_access_secret_key=ctx.obj['AWS_SECRET_ACCESS_KEY'],
                      aws_session_token=ctx.obj['AWS_SESSION_TOKEN'],
                      aws_default_region=ctx.obj['AWS_DEFAULT_REGION'],
                      debug=ctx.obj['DEBUG'])
    human_alarms, script_alarms = kinesis.remote_alarms(namespace=namespace,
                                                        prefix=prefix)

    if not no_human:
        click.echo('Human alarms.')
        if len(human_alarms) > 0:
            for alarm in human_alarms:
                click.echo(str(alarm))
        else:
            click.echo('None.')

    if not no_script:
        click.echo('Script alarms.')
        if len(script_alarms) > 0:
            for alarm in script_alarms:
                click.echo(str(alarm))
        else:
            click.echo('None.')


if __name__ == "__main__":
    main()



from .cloudwatch import CloudWatch


class EC2Instance:

    TYPE_TO_CORES = {
        "t1.micro": 1,
        "t2.micro": 1,
        "t2.small": 1,
        "t2.medium": 2,
        "t2.large": 2,
        "t2.xlarge": 4,
        "t2.2xlarge": 8,
        "m1.small": 1,
        "m1.medium": 1,
        "m1.large": 2,
        "m1.xlarge": 4,
        "m2.xlarge": 2,
        "m2.2xlarge": 4,
        "m2.4xlarge": 8,
        "m3.medium": 1,
        "m3.large": 1,
        "m3.xlarge": 2,
        "m3.2xlarge": 4,
        "m4.large": 1,
        "m4.xlarge": 2,
        "m4.2xlarge": 4,
        "m4.4xlarge": 8,
        "m4.10xlarge": 20,
        "m4.16xlarge": 32,
        "c1.medium": 2,
        "c1.xlarge": 8,
        "cc2.8xlarge": 16,
        "cg1.4xlarge": 8,
        "cr1.8xlarge": 16,
        "c3.large": 1,
        "c3.xlarge": 2,
        "c3.2xlarge": 4,
        "c3.4xlarge": 8,
        "c3.8xlarge": 16,
        "c4.large": 1,
        "c4.xlarge": 2,
        "c4.2xlarge": 4,
        "c4.4xlarge": 8,
        "c4.8xlarge": 18,
        "hi1.4xlarge": 8,
        "hs1.8xlarge": 8,
        "g2.2xlarge": 16,
        "x1.16xlarge": 32,
        "x1.32xlarge": 64,
        "r4.large": 1,
        "r4.xlarge": 2,
        "r4.2xlarge": 4,
        "r4.4xlarge": 8,
        "r4.8xlarge": 16,
        "r4.16xlarge": 32,
        "r3.large": 1,
        "r3.xlarge": 2,
        "r3.2xlarge": 4,
        "r3.4xlarge": 8,
        "r3.8xlarge": 16,
        "p2.xlarge": 2,
        "p2.8xlarge": 16,
        "p2.16xlarge": 32,
        "i3.large": 1,
        "i3.xlarge": 2,
        "i3.2xlarge": 4,
        "i3.4xlarge": 8,
        "i3.8xlarge": 16,
        "i3.16xlarge": 32,
        "i2.xlarge": 2,
        "i2.2xlarge": 4,
        "i2.4xlarge": 8,
        "i2.8xlarge": 16,
        "d2.xlarge": 2,
        "d2.2xlarge": 4,
        "d2.4xlarge": 8,
        "d2.8xlarge": 18
    }

    def __init__(self, client, info):
        self.client = client
        find_name_in_tags = filter(lambda tag: tag['Key'] == 'Name', info.get('Tags')) # noqa E501
        self.instance_id = info.get('InstanceId')
        self.type = info.get('InstanceType')
        if find_name_in_tags and len(find_name_in_tags) > 0:
            self.name = find_name_in_tags[0]['Value']
        else:
            self.name = self.instance_id

    def __str__(self):
        return '(EC2Instance) Name: %s | Type: %s' % (self.name, self.type)

    def default_dimension_name(self):
        return 'InstanceId'

    def default_dimension_value(self):
        return self.instance_id

    def default_dimensions(self):
        return [dict(Name=self.default_dimension_name(),
                     Value=self.default_dimension_value())]

    def cpu_utilization_threshold_modifier(self, threshold):
        if self.type not in self.TYPE_TO_CORES:
            raise Exception('Unknown instance type %s.' % self.type)
        cores = self.TYPE_TO_CORES[self.type]
        return threshold / cores

    def dict(self):
        return {'InstanceId': self.instance_id,
                'Type': self.type}


class EC2(CloudWatch, object):

    DEFAULT_NAMESPACE = 'AWS/EC2'
    ALARM_NAME_PREFIX = 'EC2'

    def __init__(self, aws_access_key_id=None, aws_access_secret_key=None,
                 aws_session_token=None, aws_default_region=None, debug=None):
        super(EC2, self).__init__(aws_access_key_id=aws_access_key_id,
                                  aws_access_secret_key=aws_access_secret_key,
                                  aws_session_token=aws_session_token,
                                  aws_default_region=aws_default_region,
                                  debug=debug)
        self.client = self.session.client('ec2')

    def _describe_instances(self):
        ec2s = []
        pager = self.client.get_paginator('describe_instances')
        for page in pager.paginate():
            for r in page['Reservations']:
                for i in r['Instances']:
                    ec2s.append(EC2Instance(self.client, i))
        return ec2s

    def list(self):
        return self._describe_instances()

    def remote_alarms(self, namespace=DEFAULT_NAMESPACE,
                      prefix=ALARM_NAME_PREFIX):
        namespace = namespace or EC2.DEFAULT_NAMESPACE
        prefix = prefix or EC2.ALARM_NAME_PREFIX
        return super(EC2, self).remote_alarms(namespace=namespace,
                                              prefix=prefix)

    def create(self, objects, namespace=DEFAULT_NAMESPACE,
               prefix=ALARM_NAME_PREFIX, default=None, only=None,
               exclude=None, sns={}, simulate=False):
        if exclude is not None and only is not None:
            raise "Exlude and Only option are mutually exclusive."

        instances = self._describe_instances()
        super(EC2, self).create(instances=instances,
                                objects=objects,
                                namespace=namespace,
                                prefix=prefix,
                                default=default,
                                only=only,
                                exclude=exclude,
                                sns=sns,
                                simulate=simulate)



from .cloudwatch import CloudWatch


class ELBInstance:
    def __init__(self, client, info):
        self.client = client
        self.arn = info.get('LoadBalancerArn')
        self.name = info.get('LoadBalancerName')

    def __str__(self):
        return '(ELBInstance) Name: %s' % self.name

    def default_dimension_name(self):
        return 'LoadBalancerName'

    def default_dimension_value(self):
        return self.name

    def default_dimensions(self):
        return [dict(Name=self.default_dimension_name(),
                     Value=self.default_dimension_value())]

    def dict(self):
        return {'LoadBalancerArn': self.arn,
                'LoadBalancerName': self.name}


class ELB(CloudWatch, object):

    DEFAULT_NAMESPACE = 'AWS/ELB'
    ALARM_NAME_PREFIX = 'ELB'

    def __init__(self, aws_access_key_id=None, aws_access_secret_key=None,
                 aws_session_token=None, aws_default_region=None, debug=None):
        super(ELB, self).__init__(aws_access_key_id=aws_access_key_id,
                                  aws_access_secret_key=aws_access_secret_key,
                                  aws_session_token=aws_session_token,
                                  aws_default_region=aws_default_region,
                                  debug=debug)
        self.client = self.session.client('elbv2')

    def _describe_load_balancers(self):
        elbs = []
        pager = self.client.get_paginator('describe_load_balancers')
        for page in pager.paginate():
            for i in page['LoadBalancers']:
                elbs.append(ELBInstance(self.client, i))
        return elbs

    def list(self):
        return self._describe_load_balancers()

    def remote_alarms(self, namespace=DEFAULT_NAMESPACE,
                      prefix=ALARM_NAME_PREFIX):
        namespace = namespace or ELB.DEFAULT_NAMESPACE
        prefix = prefix or ELB.ALARM_NAME_PREFIX
        return super(ELB, self).remote_alarms(namespace=namespace,
                                              prefix=prefix)

    def create(self, objects, namespace=DEFAULT_NAMESPACE,
               prefix=ALARM_NAME_PREFIX, default=None, only=None,
               exclude=None, sns={}, simulate=False):
        if exclude is not None and only is not None:
            raise "Exlude and Only option are mutually exclusive."

        instances = self._describe_load_balancers()
        super(ELB, self).create(instances=instances,
                                objects=objects,
                                namespace=namespace,
                                prefix=prefix,
                                default=default,
                                only=only,
                                exclude=exclude,
                                sns=sns,
                                simulate=simulate)



class Alarm(object):

    CREATED_BY_SCRIPT_STR = 'Created by Script'

    def __init__(self, name, namespace=None, info={}):
        self.update(name=name, namespace=namespace, info=info)
        self.resolved = False
        self.is_human = False

    def update(self, name=None, namespace=None, info={}):
        self.name = info.get('AlarmName') or name
        self.namespace = info.get('Namespace') or namespace
        self.arn = info.get('AlarmArn')
        self.metric_name = info.get('MetricName')
        self.statistic = info.get('Statistic')
        self.comparison_operator = info.get('ComparisonOperator')
        self.evaluation_periods = info.get('EvaluationPeriods')
        self.period = info.get('Period')
        self.threshold = info.get('Threshold')
        self.alarm_actions = info.get('AlarmActions')
        self.ok_actions = info.get('OKActions')
        self.dimensions = info.get('Dimensions') or []
        self.description = info.get('AlarmDescription') or self.CREATED_BY_SCRIPT_STR  # noqa E501
        self.treat_missing_data = info.get('TreatMissingData') or 'missing'

    def is_valid(self):
        cond1 = self.name
        cond2 = self.metric_name
        cond3 = self.period
        cond4 = self.evaluation_periods
        cond5 = self.statistic
        cond6 = self.dimensions
        cond7 = self.description
        cond8 = ((self.alarm_actions and len(self.alarm_actions)) > 0 or (self.ok_actions and len(self.ok_actions) > 0))  # noqa E501
        return all([cond1, cond2, cond3, cond4, cond5, cond6, cond7, cond8])

    def is_created_by_script(self):
        cond1 = self.description
        cond2 = self.description == self.CREATED_BY_SCRIPT_STR
        return all([cond1, cond2])

    def dict(self):
        return self.all_dict()

    def all_dict(self):
        return {'AlarmName': self.name,
                'Namespace': self.namespace,
                'MetricName': self.metric_name,
                'Statistic': self.statistic,
                'ComparisonOperator': self.comparison_operator,
                'EvaluationPeriods': self.evaluation_periods,
                'Period': self.period,
                'Threshold': self.threshold,
                'AlarmActions': self.alarm_actions or [],
                'OKActions': self.ok_actions or [],
                'Dimensions': self.dimensions,
                'AlarmDescription': self.description,
                'TreatMissingData': self.treat_missing_data}

    def __str__(self):
        return ('{} ('
                'Namespace: {}, '
                'MetricName: {}, '
                'Statistic: {}, '
                'ComparisonOperator: {}, '
                'EvaluationPeriods: {}, '
                'Period: {}, '
                'Threshold: {}, '
                'Created by script: {}').format(self.name,
                                                self.namespace or None,
                                                self.metric_name,
                                                self.statistic,
                                                self.comparison_operator,
                                                self.evaluation_periods,
                                                self.period,
                                                self.threshold,
                                                self.is_created_by_script())



from .cloudwatch import CloudWatch


class KinesisInstance:
    def __init__(self, client, name):
        self.client = client
        self.name = name

    def __str__(self):
        return '(KinesisInstance) Name: %s' % self.name

    def default_dimension_name(self):
        return 'StreamName'

    def default_dimension_value(self):
        return self.name

    def default_dimensions(self):
        return [dict(Name=self.default_dimension_name(),
                     Value=self.default_dimension_value())]

    def dict(self):
        return {'StreamName': self.name}


class Kinesis(CloudWatch, object):

    DEFAULT_NAMESPACE = 'AWS/Kinesis'
    ALARM_NAME_PREFIX = 'Kinesis'

    def __init__(self, aws_access_key_id=None, aws_access_secret_key=None,
                 aws_session_token=None, aws_default_region=None, debug=None):
        super(Kinesis, self).__init__(aws_access_key_id=aws_access_key_id,
                                      aws_access_secret_key=aws_access_secret_key,  # noqa E501
                                      aws_session_token=aws_session_token,
                                      aws_default_region=aws_default_region,
                                      debug=debug)
        self.client = self.session.client('kinesis')

    def _list_streams(self):
        response = self.client.list_streams(Limit=1000)
        return [KinesisInstance(self.client, name) for name in response['StreamNames']]  # noqa E501

    def list(self):
        return self._list_streams()

    def remote_alarms(self, namespace=DEFAULT_NAMESPACE,
                      prefix=ALARM_NAME_PREFIX):
        namespace = namespace or Kinesis.DEFAULT_NAMESPACE
        prefix = prefix or Kinesis.ALARM_NAME_PREFIX
        return super(Kinesis, self).remote_alarms(namespace=namespace,
                                                  prefix=prefix)

    def create(self, objects, namespace=DEFAULT_NAMESPACE,
               prefix=ALARM_NAME_PREFIX, default=None, only=None,
               exclude=None, sns={}, simulate=False):

        instances = self._list_streams()
        super(Kinesis, self).create(instances=instances,
                                    objects=objects,
                                    namespace=namespace,
                                    prefix=prefix,
                                    default=default,
                                    only=only,
                                    exclude=exclude,
                                    sns=sns,
                                    simulate=simulate)



from .cloudwatch import CloudWatch


class PGBouncerInstance:
    def __init__(self, info):
        self.name = info

    def __str__(self):
        return '(PGBouncer Instance) Name: %s' % self.name

    def default_dimension_name(self):
        return 'Workers'

    def default_dimension_value(self):
        return self.name.partition('.')[-1]

    def default_dimensions(self):
        return [dict(Name=self.default_dimension_name(),
                     Value=self.default_dimension_value()),
                dict(Name='PGBouncer',
                     Value=self.name)]

    def dict(self):
        return {'PGBouncerIdentifier': self.default_dimension_value()}


class PGBouncer(CloudWatch, object):

    DEFAULT_NAMESPACE = 'ISC'
    ALARM_NAME_PREFIX = 'PGBOUNCER'

    def __init__(self, aws_access_key_id=None, aws_access_secret_key=None,
                 aws_session_token=None, aws_default_region=None, debug=None):
        super(PGBouncer, self).__init__(aws_access_key_id=aws_access_key_id,
                                        aws_access_secret_key=aws_access_secret_key, # noqa E501
                                        aws_session_token=aws_session_token,
                                        aws_default_region=aws_default_region,
                                        debug=debug)

    def _describe_pgbouncer_instances(self):
        response = self.instance_list()
        return [PGBouncerInstance(i) for i in response]

    def list(self):
        return self._describe_pgbouncer_instances()

    def instance_list(self, namespace=DEFAULT_NAMESPACE,
                      prefix=ALARM_NAME_PREFIX):
        human_alarms, script_alarms = self.remote_alarms(namespace, prefix)
        alarms = human_alarms + script_alarms
        all_hosts = set()
        map(lambda alarm: all_hosts.add(
            PGBouncerInstance(alarm.name.split('/')[1])), alarms)
        return list(all_hosts)

    def remote_alarms(self, namespace=DEFAULT_NAMESPACE,
                      prefix=ALARM_NAME_PREFIX):
        namespace = namespace or PGBouncer.DEFAULT_NAMESPACE
        prefix = prefix or PGBouncer.ALARM_NAME_PREFIX
        return super(PGBouncer, self).remote_alarms(namespace=namespace,
                                                    prefix=prefix)

    def create(self, objects, namespace=DEFAULT_NAMESPACE,
               prefix=ALARM_NAME_PREFIX, default=None, only=None,
               exclude=None, sns={}, simulate=False):
        if exclude is not None and only is not None:
            raise "Exclude and Only option are mutually exclusive."

        instances = self.instance_list()
        super(PGBouncer, self).create(instances=instances,
                                      objects=objects,
                                      namespace=namespace,
                                      prefix=prefix,
                                      default=default,
                                      only=only,
                                      exclude=exclude,
                                      sns=sns,
                                      simulate=simulate)



def to_underscore(name):
    import re
    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()


def json_serializer(obj):
    """Default JSON serializer."""
    import calendar
    import datetime

    if isinstance(obj, datetime.datetime):
        if obj.utcoffset() is not None:
            obj = obj - obj.utcoffset()
        millis = int(
            calendar.timegm(obj.timetuple()) * 1000 +
            obj.microsecond / 1000
        )
        return millis

    if isinstance(obj, set):
        return list(obj)

    raise TypeError('Not sure how to serialize %s' % (obj,))


def filter_only(iterable, only):
    out = []
    for info in only:
        for i in iterable:
            if info['regexp'].search(i.dict().get(info['key'])):
                out.append(i)
    return out


def filter_exclude(iterable, only):
    out = []
    for info in only:
        for i in iterable:
            if not info['regexp'].search(i.dict().get(info['key'])):
                out.append(i)
    return out


def extract_alarm_actions(default, params, sns, action):
    actions_sns = []
    if params.get(action):
        for _sns in params.get(action):
            if _sns.startswith('arn:'):
                actions_sns.append(_sns)
            elif sns.get(_sns):
                actions_sns.append(sns.get(_sns))

    if len(actions_sns) < 1 and sns.get('default'):
        _sns = sns.get('default')
        if _sns and _sns.startswith('arn:'):
            actions_sns.append(_sns)

    return actions_sns


def resolved_dict(name, instance, original, default, namespace=None,
                  prefix=None, sns={}):
    import copy
    original = copy.deepcopy(original)
    default = copy.deepcopy(default)

    default_params = {}
    default_all_params = {}
    default_instance_params = {}

    if default.get('all'):
        default_all_params = default.get('all').dict()
        for k in default_all_params.keys():
            if not default_all_params[k]:
                del default_all_params[k]

    filtered_instance = list(filter(lambda i: i in instance.name, default.keys()))  # noqa E501
    instance_name = instance.name
    if len(filtered_instance) > 0:
        instance_name = max(filtered_instance, key=len)

    if default.get(instance_name):
        default_instance_params = default.get(instance_name).dict()
        for k in list(default_instance_params):
            if not default_instance_params[k]:
                del default_instance_params[k]

    default_params = dict(default_all_params, **default_instance_params)

    params = {}

    params = dict(original, **default_params)

    params['AlarmActions'] = extract_alarm_actions(default, params, sns, 'AlarmActions')  # noqa E501
    params['OKActions'] = extract_alarm_actions(default, params, sns, 'OKActions')  # noqa E501
    params['InsufficientDataActions'] = extract_alarm_actions(default, params, sns, 'InsufficientDataActions')  # noqa E501

    if prefix is not None:
        params['AlarmName'] = '{}/{}/{}'.format(prefix, instance.name, name)

    if params.get('Dimensions') and len(params.get('Dimensions')) > 0:
        for dim in params['Dimensions']:
            value = dim.get('Value')
            if value:
                dim['Value'] = instance.dict().get(value)
    else:
        params['Dimensions'] = instance.default_dimensions()

    for attr in params:
        method_name = '%s_%s_modifier' % (to_underscore(params['MetricName']), attr.lower())  # noqa E501
        if(hasattr(instance, method_name)):
            params[attr] = getattr(instance, method_name)(params[attr])

    return params



import boto3


class Client:

    def __init__(self, aws_access_key_id=None, aws_access_secret_key=None,
                 aws_session_token=None, aws_default_region=None, debug=None):
        self.aws_access_key_id = aws_access_key_id
        self.aws_access_secret_key = aws_access_secret_key
        self.aws_session_token = aws_session_token
        self.aws_default_region = aws_default_region
        self.debug = debug
        self.session = boto3.Session(aws_access_key_id=aws_access_key_id,
                                     aws_secret_access_key=aws_access_secret_key,  # noqa E501
                                     aws_session_token=aws_session_token,
                                     region_name=aws_default_region)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'Click>=6.7',
    'boto3>=1.4.3',
    'dictdiffer>=0.6.1',
    'PyYAML>=3.12'
]

test_requirements = [
    # TODO: put package test requirements here
]

setup(
    name='cwam',
    version='1.0.0',
    description="Easy way to create default CloudWatch Alarms.",
    long_description=readme + '\n\n' + history,
    author="Quentin Rousseau",
    author_email='quentin@instacart.com',
    url='https://github.com/instacart/cwam',
    packages=[
        'cwam',
    ],
    package_dir={'cwam':
                 'cwam'},
    entry_points={
        'console_scripts': [
            'cwam=cwam.cli:main'
        ]
    },
    include_package_data=True,
    install_requires=requirements,
    license="MIT license",
    zip_safe=False,
    keywords='cloudwatch',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development',
        'Topic :: System :: Monitoring'
    ],
    test_suite='tests',
    tests_require=test_requirements
)



class DefaultAlarm(object):

    def __init__(self, namespace=None, info={}):
        self.namespace = info.get('Namespace') or namespace
        self.evaluation_periods = info.get('EvaluationPeriods')
        self.period = info.get('Period')
        self.alarm_actions = info.get('AlarmActions') or []
        self.ok_actions = info.get('OKActions') or []
        self.dimensions = info.get('Dimensions') or []

    def dict(self):
        return {'AlarmActions': self.alarm_actions,
                'EvaluationPeriods': self.evaluation_periods,
                'Period': self.period,
                'OKActions': self.ok_actions,
                'Dimensions': self.dimensions}

    def __str__(self):
        return ('(AlarmActions: {}, '
                'EvaluationPeriods: {}, '
                'Period: {}, '
                'OKActions: {}, '
                'Dimensions: {})').format(self.alarm_actions,
                                          self.evaluation_periods,
                                          self.period,
                                          self.ok_actions,
                                          self.dimensions)
